/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

const fs = require('fs')
const axios = require('axios')
const chalk = require('chalk')
const jsonfile = require('jsonfile')
const FormData = require('form-data')

const API = require('../util/API')
const output = require('../output')
const runHelpers = require('../run-helpers')
const Module = require('../containers/Module')

module.exports = function (name) {
  API.checkSignedIn()

  Module.getManifest(function (manifest) {
    if (!('name' in manifest) || !manifest.name || typeof manifest.name !== 'string' || !manifest.name.length) {
      output.err('Please set a valid value for key `name` in module.json.')
      process.exit(1)
    }

    name = name || manifest.name
    if (name && name !== manifest.name) {
      if (!name.match(/^[a-z0-9-]+\/[a-z0-9-]+$/)) {
        output.err('Module name should match org/name pattern and only contain lowercase letters, numbers and dashes.')
        process.exit(1)
      }
      output.warn('Ignoring the module name set in module.json')
    }

    if (!name.match(/^[a-z0-9-/]+$/)) {
      output.err('Module name should only contain lowercase letters, numbers and dashes.')
      process.exit(1)
    }

    runHelpers.startChildProcess('build', function (error, exitCode) {
      if (error || exitCode > 0) {
        output.err('Build script exited with non-zero exit code, cancelling publish action')
        process.exit(1)
      }

      let originalManifest = null
      if (name !== manifest.name) {
        manifest.name = name
        originalManifest = fs.readFileSync('module.json', 'utf8')
        jsonfile.writeFileSync('module.json', manifest)
      }

      runHelpers.createZipArchive('.', 'Pack', function (zipFile, size, unlinkFn) {
        if (originalManifest) {
          fs.writeFileSync('module.json', originalManifest, 'utf8')
        }

        const spinner = output.wait(runHelpers.logSyncMessage('Uploading and building module (' + size + ')', true))

        const data = new FormData(null)
        data.append('file', fs.createReadStream(zipFile))
        axios.post('https://monkey.internal.includable.com/upload', data, {
          headers: data.getHeaders(),
          maxContentLength: Infinity,
          maxBodyLength: Infinity
        })
          .then(function (response) {
            spinner(true)

            if (response.data && response.data.log) {
              response.data.log.forEach(line => output.indented('     ' + chalk.dim(chalk.bold('→')) + ' ' + line))
            }

            const spinner2 = output.wait(runHelpers.logSyncMessage('Creating deployment', true))
            API.post('/modules/publish', { registry_id: response.data.id })
              .then(function (response) {
                setTimeout(function () {
                  spinner2(true)
                  output.success('\n' + response.data.data.message)
                }, 5000)
                unlinkFn()
              })
              .catch(function (response) {
                spinner2(response.response.data.error || 'Network error')
                unlinkFn()
                process.exit(1)
              })
          })
          .catch(function (response) {
            spinner((response.response && response.response.data.error) || response || 'Network error')
            console.log(response.response)
            unlinkFn()
            process.exit(1)
          })
      })
    })
  })
}
