/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

'use strict'

const jsonfile = require('jsonfile')
const mkdirp = require('mkdirp')
const path = require('path')
const fs = require('fs')

var ContainerConfig = {}

/**
 * Validate a config object.
 *
 * @param {ContainerConfigObject} config
 * @returns {boolean}
 */
ContainerConfig.validate = function (config) {
  config.containerToken = config.containerToken || config.container_token

  if ('containerToken' in config) {
    ContainerConfig.setup(config)
    return true
  }

  return false
}

/**
 * Set up the app's globals using config values.
 *
 * @param {ContainerConfigObject} config
 */
ContainerConfig.setup = function (config) {
  if ('containerEndpoint' in config) {
    global.containerEndpoint = config.containerEndpoint
  }
  if ('version' in config) {
    global.apiVersion = config.version
  }
  if ('stability' in config) {
    global.apiStability = config.stability
  }
  if ('containerEndpointCheckSSL' in config && !config.containerEndpointCheckSSL) {
    process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
  }
}

/**
 * Get the correct (relative) path for a container config file.
 *
 * @param {string} name
 * @returns {string}
 */
ContainerConfig.getPath = function (name) {
  return path.join('.inc', 'containers', name + '.json')
}

/**
 * Read a container config file. If the file with the supplied container name doesn't exist,
 * this falls back to the 'default' container, if that one exists.
 *
 * @param {string} name
 * @param {function(Error, ContainerConfigObject)} cb
 */
ContainerConfig.read = function (name, cb) {
  jsonfile.readFile(ContainerConfig.getPath(name), function (err, config) {
    // Container file exists and is a JSON object, so check if it is valid
    if (!err && typeof config === 'object' && ContainerConfig.validate(config)) {
      // Yep, return in callback!
      cb(null, config)
      return
    }

    // Try default file if in `beta` mode
    if (name === 'beta') {
      ContainerConfig.read('default', cb)
      return
    }

    // No valid container found
    cb(err || new Error('Container config not found or invalid'), null)
  })
}

/**
 * Write to the config file for a specific container.
 *
 * @param {string} name
 * @param {object} content
 */
ContainerConfig.write = function (name, content) {
  // Ensure the containers directory exists
  mkdirp(path.join('.inc', 'containers'), function () {
    // Write JSON file
    jsonfile.writeFile(ContainerConfig.getPath(name), content)
    // Write package version file
    if (!fs.existsSync(path.join('.inc', 'version'))) {
      fs.writeFileSync(path.join('.inc', 'version'), global.containerVersion.toString(), 'utf8')
    }
  })
}

module.exports = ContainerConfig
