/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

'use strict'

const ora = require('ora')
const chalk = require('chalk')
const moment = require('moment')
const format = require('util').format
const slice = Array.prototype.slice

var theme = {
  success: chalk.green,
  warn: chalk.yellow,
  yellow: chalk.yellow,
  err: chalk.red,
  errSimple: chalk.red,
  errEmphasis: chalk.red.underline,
  info: chalk.underline.white,
  offset: chalk.italic,
  notice: chalk.underline,
  gray: chalk.gray,
  indented: chalk.default
}

var themeChars = {
  notice: '',
  log: '•',
  warn: 'Warning: ',
  err: '• ',
  errEmphasis: '',
  errSimple: '',
  success: '',
  offset: '',
  info: '',
  yellow: '',
  gray: '',
  indented: '  '
}

var activeSpinner = null
var backlog = []

function processBacklog () {
  for (var x in backlog) {
    var action = backlog[x]
    exports[action[0]].apply(exports, action[1])
  }
  backlog = []
}

function log () {
  var args = slice.call(arguments)
  var level = args.shift()

  args[0] = args[0] ? themeChars[level] + args[0] : ''
  args[0] = args[0] ? theme[level](args[0]) : ''
  console.log.apply(console, args)
}

function write () {
  var out = process.stdout
  var args = slice.call(arguments)
  var level = args.shift()

  out.write(theme[level](format.apply(null, [themeChars[level]].concat(args))))
}

exports.debug = function () {
  if (!global.verbose) {
    return
  }

  exports.log.apply(exports, arguments)
}

exports.log = function () {
  if (activeSpinner) {
    backlog.push(['log', arguments])
    return
  }

  console.log.apply(console, [themeChars.log].concat(slice.call(arguments)))
}

exports.write = function (str) {
  process.stdout.write(str)
}

exports.newline = function () {
  process.stdout.write('\n')
}

exports.wait = function (msg) {
  var spinner = ora({
    text: chalk.gray(msg),
    color: 'green'
  })

  activeSpinner = spinner

  spinner.start()

  return function (result) {
    if (result === undefined) {
      spinner.stop()
      process.stdout.write('\u001B[2K')
    } else if (result === true) {
      spinner.succeed()
    } else {
      spinner.fail(result)
    }

    activeSpinner = null
    processBacklog()
  }
}

exports.logComponent = function (component, color, message) {
  message.split('\n').map(function (string) {
    if (string.length) {
      exports.logComponentLine(component, color, string)
    }
  })
}

exports.logComponentLine = function (component, color, message) {
  var str = chalk.bold(chalk[color](component))
  str += ' ' + message
  str += ' ' + chalk.dim(moment().format('hh:mm:ss'))

  return this.log(str)
}

// will be overwritten later, only here for syntax completion
exports.notice = exports.info = exports.success = exports.gray =
  exports.indented = exports.yellow = exports.err = exports.errSimple = exports.warn = function () {}

Object.keys(theme).forEach(function (level) {
  exports[level] = function () {
    log.apply(null, [level].concat(slice.call(arguments)))
  }

  var c = level[0].toUpperCase()
  exports['write' + c + level.slice(1)] = function () {
    write.apply(null, [level].concat(slice.call(arguments)))
  }
})
