/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

'use strict'

const output = require('./output')
const Container = require('./containers/Container')
const autoreload = require('./plugins/Autoreload')
const Topics = require('@includable/topics')
const fs = require('fs')
const tmp = require('tmp')
const path = require('path')
const chalk = require('chalk')
const moment = require('moment')
const request = require('./util').request
const child = require('./util/ChildProcess')
const Module = require('./containers/Module')
const archiver = require('archiver')
const filesize = require('file-size')

module.exports = {
  uploadFile: function (containerToken, event, path, cb, spinner) {
    const r = request(containerToken + '/fs', {
      'action': event,
      'name': path
    }, {
      'Content-Type': 'multipart/form-data'
    })

    if (event === 'add' || event === 'change' || event === 'zipBall') {
      r.attach('file', path)
    }

    const self = this
    return r.end(function (response) {
      const func = spinner || output.err
      if ('error' in response && response.error) {
        Container.clean()
        func(response.error)
        process.exit()
      }

      if (response.body && response.body === 'Error: invalid container') {
        Container.clean()
        func(
          'Container ' + containerToken + ' does not exist anymore. ' +
          'We cleaned your local environment, so that you can start with a clean container.')
        process.exit()
      }

      autoreload(containerToken, path, event)

      if (cb) {
        cb(response.body || '')
        return
      }
      if (!response.body) {
        return
      }
      if (response.body.indexOf('Error:') === 0) {
        output.err(response.body)
        return
      }
      self.logSyncMessage(response.body)
    })
  },

  logMessage: function (data, ret) {
    let message =
      chalk.bold(chalk[data.color](data.component)) + ' ' +
      data.message

    if ('time' in data) {
      if (data.time === 'now') {
        data.time = (new Date()).getTime() / 1000
      }
      message += ' ' + chalk.dim(moment(data.time * 1000).format('hh:mm:ss'))
    }

    if (ret) {
      return message
    }

    return output.log(message)
  },

  logSyncMessage: function (message, ret, component) {
    return this.logMessage({
      component: component || 'Sync',
      color: 'green',
      message: message,
      time: 'now'
    }, !!ret)
  },

  createInitialZip: function (dir, containerToken, envr, cb) {
    const self = this
    output.debug('Starting zip archive creation')
    this.createZipArchive(dir, 'Sync', function (zipName, size, unlinkFn) {
      const stopSpinner = output.wait(self.logSyncMessage('Uploading initial zipball (' + size + ')', true))
      try {
        self.uploadFile(containerToken, 'zipBall', zipName, function (responseBody) {
          unlinkFn()
          if (responseBody.indexOf('OK') !== 0) {
            stopSpinner(responseBody || 'Could not connect to server (empty response)')
            process.exit()
          }
          stopSpinner(true)
          cb()
        }, stopSpinner)
      } catch (e) {
        unlinkFn()
        stopSpinner(e.message)
      }
    })
  },

  createZipArchive: function (dir, label, cb) {
    if (fs.existsSync(path.join(dir, 'vendor'))) {
      output.warn('Ignoring directory \'vendor\'')
    }
    if (fs.existsSync(path.join(dir, 'node_modules'))) {
      output.warn('Ignoring directory \'node_modules\'')
    }
    if (fs.existsSync(path.join(dir, 'web/node_modules'))) {
      output.warn('Ignoring directory \'web/node_modules\'')
    }

    const zipName = tmp.tmpNameSync({})
    const spinner = output.wait(this.logSyncMessage('Creating zipball', true, label))
    const zipFile = fs.createWriteStream(zipName)

    zipFile.on('close', function () {
      // Upload file to sandbox
      fs.stat(zipName, function (err, stat) {
        if (err) {
          spinner('Error creating zipball')
          process.exit()
        }
        spinner(true)

        const size = filesize(stat.size).human('si')
        cb(zipName, size, function () {
          fs.unlink(zipName, function () {})
        })
      })
    })

    const archive = archiver('zip', {
      store: true
    })
    archive.on('error', function (err) {
      output.err(err)
    })
    archive.pipe(zipFile)
    archive.glob(path.join(dir, '**'), {
      ignore: [
        '.inc/**', '.git/**', 'node_modules', 'web/node_modules', 'node_modules/**',
        'web/node_modules/**', 'vendor/**', '.gitignore', '.eslintrc', '.jshintrc'
      ]
    })
    archive.finalize()
  },

  startChildProcess: function (type, callback, basePath) {
    Module.getManifest(function (manifest) {
      // Get scripts object
      const scripts = manifest.getDevOption('scripts')
      if (!scripts || typeof scripts !== 'object' || !(type in scripts)) {
        return callback ? callback(null, 0) : false
      }

      // Cast to array if script is string
      if (typeof scripts[type] === 'string') {
        scripts[type] = [scripts[type]]
      }

      if (!(type in scripts) || !('length' in scripts[type])) {
        return callback ? callback(null, 0) : false
      }

      let count = 0
      for (let x in scripts[type]) {
        if (!scripts[type].hasOwnProperty(x)) {
          continue
        }
        const script = scripts[type][x]
        child.run(script, callback, basePath)
        count++
      }

      return !!count
    }, basePath)
  },

  startWebSocket: function (containerToken) {
    Topics.sub('SADEV:' + containerToken, (log) => {
      if (!('component' in log)) {
        log.component = 'Logger'
        log.color = 'yellow'
      }
      if (!('color' in log)) {
        log.color = 'blue'
      }

      this.logMessage(log)
    })
  }
}
