/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

const axios = require('axios')
const netrc = require('netrc')
const FormData = require('form-data')
const queryString = require('query-string')
const pkg = require('../../package.json')
const output = require('../output')

const API = axios.create({
  maxRedirects: 0,
  baseURL: 'https://dashboard.includable.com/dashboard/v1',
  headers: {
    'Accept': 'application/json',
    'User-Agent': 'IncludableCLI ' + pkg.name + '/' + pkg.version,
    'X-Client': pkg.name + '/' + pkg.version,
    'X-Client-Version': pkg.version,
    'X-API-Version': global.apiVersion,
    'X-API-Stability': global.apiStability
  },
  transformRequest: [function (data, headers) {
    const token = API.getToken()
    if (token) {
      headers['Authorization'] = 'Bearer ' + token
    }

    if (typeof data === 'object' && !(data instanceof FormData)) {
      data = queryString.stringify(data)
    }

    return data
  }]
})

/**
 * Get the user's Includable token from the .netrc file.
 *
 * @returns {string|null}
 */
API.getToken = function () {
  if ('INCLUDABLE_TOKEN' in process.env && process.env.INCLUDABLE_TOKEN) {
    return process.env.INCLUDABLE_TOKEN
  }

  let nrc = netrc(null)
  if (nrc && 'dashboard.includable.com' in nrc && 'password' in nrc['dashboard.includable.com']) {
    return nrc['dashboard.includable.com'].password
  }

  return null
}

/**
 * Check if the user is signed in. If not, output error and kill the process.
 */
API.checkSignedIn = function () {
  const token = API.getToken()
  if (!token) {
    output.errSimple('You are not signed in. First run `inc login` before running this command.')
    process.exit(1)
  }
}

/**
 * Save the user's token in a .netrc file.
 *
 * @param {string} token
 */
API.setToken = function (token) {
  let nrc = netrc(null)
  if (!nrc) {
    throw new Error('Could not load netrc file.')
  }

  nrc['dashboard.includable.com'] = {
    login: 'x-oauth',
    password: token
  }

  netrc.save(nrc)
}

module.exports = API
